@@ -1,6 +1,7 @@
 from __future__ import print_function
 
 import argparse
+import logging
 import sys
 import os.path
 from glob import glob
@@ -72,6 +73,13 @@ def main():
     args = parser.parse_args()
     env_yaml = get_env_yaml(args.build_config_yaml, args.product_yaml)
 
+    build_root = os.path.dirname(args.build_config_yaml)
+
+    logfile = "{build_root}/{product}/control_profiles.log".format(
+            build_root=build_root,
+            product=env_yaml["product"])
+    logging.basicConfig(filename=logfile, level=logging.INFO)
+
     if args.controls_dir:
         controls_manager = ssg.controls.ControlsManager(args.controls_dir, env_yaml)
         controls_manager.load()
