@@ -114,8 +114,13 @@ class Profile(object):
         self.refine_rules = defaultdict(list)
         self.metadata = None
         self.reference = None
+        # self.platforms is used further in the build system
+        # self.platform is merged into self.platforms
+        # it is here for backward compatibility
+        self.platforms = set()
         self.platform = None
 
+
     def read_yaml_contents(self, yaml_contents):
         self.title = required_key(yaml_contents, "title")
         del yaml_contents["title"]
@@ -126,6 +131,8 @@ class Profile(object):
         if selection_entries:
             self._parse_selections(selection_entries)
         del yaml_contents["selections"]
+        self.platforms = yaml_contents.pop("platforms", set())
+        self.platform = yaml_contents.pop("platform", None)
 
     @classmethod
     def from_yaml(cls, yaml_file, env_yaml=None):
@@ -139,7 +146,11 @@ class Profile(object):
         profile.read_yaml_contents(yaml_contents)
 
         profile.reference = yaml_contents.pop("reference", None)
-        profile.platform = yaml_contents.pop("platform", None)
+        # ensure that content of profile.platform is in profile.platforms as
+        # well
+        if profile.platform is not None:
+            profile.platforms.add(profile.platform)
+
 
         # At the moment, metadata is not used to build content
         if "metadata" in yaml_contents:
@@ -163,8 +174,8 @@ class Profile(object):
         if self.extends is not None:
             to_dump["extends"] = self.extends
 
-        if self.platform is not None:
-            to_dump["platform"] = self.platform
+        if self.platforms:
+            to_dump["platforms"] = self.platforms
 
         selections = []
         for item in self.selected:
@@ -219,13 +230,15 @@ class Profile(object):
         if self.reference:
             add_sub_element(element, "reference", escape(self.reference))
 
-        if self.platform:
-            try:
-                platform_cpe = product_cpes.get_cpe_name(self.platform)
-            except KeyError:
-                raise ValueError("Unsupported platform '%s' in profile '%s'." % (self.platform, self.id_))
-            plat = ET.SubElement(element, "platform")
-            plat.set("idref", platform_cpe)
+        if self.platforms:
+            for platform in self.platforms:
+                try:
+                    platform_cpe = product_cpes.get_cpe_name(platform)
+                except KeyError:
+                    raise ValueError(
+                        "Unsupported platform '%s' in profile '%s'." % (self.platform, self.id_))
+                plat = ET.SubElement(element, "platform")
+                plat.set("idref", platform_cpe)
 
         for selection in self.selected:
             select = ET.Element("select")
@@ -346,6 +359,7 @@ class Profile(object):
         profile.title = self.title
         profile.description = self.description
         profile.extends = self.extends
+        profile.platforms = self.platforms
         profile.platform = self.platform
         profile.selected = list(set(self.selected) - set(other.selected))
         profile.selected.sort()
@@ -807,7 +821,7 @@ class Group(object):
     """Represents XCCDF Group
     """
     ATTRIBUTES_TO_PASS_ON = (
-        "platform",
+        "platforms",
     )
 
     def __init__(self, id_):
@@ -821,6 +835,10 @@ class Group(object):
         self.values = {}
         self.groups = {}
         self.rules = {}
+        # self.platforms is used further in the build system
+        # self.platform is merged into self.platforms
+        # it is here for backward compatibility
+        self.platforms = set()
         self.platform = None
 
     @classmethod
@@ -840,6 +858,11 @@ class Group(object):
         group.conflicts = yaml_contents.pop("conflicts", [])
         group.requires = yaml_contents.pop("requires", [])
         group.platform = yaml_contents.pop("platform", None)
+        group.platforms = yaml_contents.pop("platforms", set())
+        # ensure that content of group.platform is in group.platforms as
+        # well
+        if group.platform is not None:
+            group.platforms.add(group.platform)
 
         for warning_list in group.warnings:
             if len(warning_list) != 1:
@@ -872,14 +895,15 @@ class Group(object):
         add_nondata_subelements(group, "requires", "id", self.requires)
         add_nondata_subelements(group, "conflicts", "id", self.conflicts)
 
-        if self.platform:
+        if self.platforms:
             platform_el = ET.SubElement(group, "platform")
-            try:
-                platform_cpe = product_cpes.get_cpe_name(self.platform)
-            except CPEDoesNotExist:
-                print("Unsupported platform '%s' in rule '%s'." % (self.platform, self.id_))
-                raise
-            platform_el.set("idref", platform_cpe)
+            for platform in self.platforms:
+                try:
+                    platform_cpe = product_cpes.get_cpe_name(platform)
+                except CPEDoesNotExist:
+                    print("Unsupported platform '%s' in rule '%s'." % (platform, self.id_))
+                    raise
+                platform_el.set("idref", platform_cpe)
 
         for _value in self.values.values():
             group.append(_value.to_xml_element())
@@ -953,8 +977,8 @@ class Group(object):
     def add_group(self, group):
         if group is None:
             return
-        if self.platform and not group.platform:
-            group.platform = self.platform
+        if self.platforms and not group.platforms:
+            group.platforms = self.platforms
         self.groups[group.id_] = group
         self._pass_our_properties_on_to(group)
 
@@ -966,8 +990,8 @@ class Group(object):
     def add_rule(self, rule):
         if rule is None:
             return
-        if self.platform and not rule.platform:
-            rule.platform = self.platform
+        if self.platforms and not rule.platforms:
+            rule.platforms = self.platforms
         self.rules[rule.id_] = rule
         self._pass_our_properties_on_to(rule)
 
@@ -993,11 +1017,14 @@ class Rule(object):
         "conflicts": lambda: list(),
         "requires": lambda: list(),
         "platform": lambda: None,
+        "platforms": lambda: set(),
         "inherited_platforms": lambda: list(),
         "template": lambda: None,
         "definition_location": lambda: None,
     }
 
+    PRODUCT_REFERENCES = ("stigid", "cis",)
+
     def __init__(self, id_):
         self.id_ = id_
         self.prodtype = "all"
@@ -1014,7 +1041,11 @@ class Rule(object):
         self.warnings = []
         self.requires = []
         self.conflicts = []
+        # self.platforms is used further in the build system
+        # self.platform is merged into self.platforms
+        # it is here for backward compatibility
         self.platform = None
+        self.platforms = set()
         self.inherited_platforms = [] # platforms inherited from the group
         self.template = None
 
@@ -1043,6 +1074,11 @@ class Rule(object):
             if len(warning_list) != 1:
                 raise ValueError("Only one key/value pair should exist for each dictionary")
 
+        # ensure that content of rule.platform is in rule.platforms as
+        # well
+        if rule.platform is not None:
+            rule.platforms.add(rule.platform)
+
         if yaml_contents:
             raise RuntimeError("Unparsed YAML data in '%s'.\n\n%s"
                                % (yaml_file, yaml_contents))
@@ -1086,11 +1122,9 @@ class Rule(object):
             raise RuntimeError(msg)
 
     def _get_product_only_references(self):
-        PRODUCT_REFERENCES = ("stigid",)
-
         product_references = dict()
 
-        for ref in PRODUCT_REFERENCES:
+        for ref in Rule.PRODUCT_REFERENCES:
             start = "{0}@".format(ref)
             for gref, gval in self.references.items():
                 if ref == gref or gref.startswith(start):
@@ -1120,6 +1154,13 @@ class Rule(object):
         general_references = self.references.copy()
         for todel in product_references:
             general_references.pop(todel)
+        for ref in Rule.PRODUCT_REFERENCES:
+            if ref in general_references:
+                msg = "Unexpected reference identifier ({0}) without "
+                msg += "product qualifier ({0}@{1}) while building rule "
+                msg += "{2}"
+                msg = msg.format(ref, product, self.id_)
+                raise ValueError(msg)
 
         to_set = dict(
             identifiers=(self.identifiers, False),
@@ -1306,14 +1347,15 @@ class Rule(object):
         add_nondata_subelements(rule, "requires", "id", self.requires)
         add_nondata_subelements(rule, "conflicts", "id", self.conflicts)
 
-        if self.platform:
-            platform_el = ET.SubElement(rule, "platform")
-            try:
-                platform_cpe = product_cpes.get_cpe_name(self.platform)
-            except CPEDoesNotExist:
-                print("Unsupported platform '%s' in rule '%s'." % (self.platform, self.id_))
-                raise
-            platform_el.set("idref", platform_cpe)
+        if self.platforms:
+            for platform in self.platforms:
+                platform_el = ET.SubElement(rule, "platform")
+                try:
+                    platform_cpe = product_cpes.get_cpe_name(platform)
+                except CPEDoesNotExist:
+                    print("Unsupported platform '%s' in rule '%s'." % (platform, self.id_))
+                    raise
+                platform_el.set("idref", platform_cpe)
 
         return rule
 
@@ -1463,7 +1505,8 @@ class BuildLoader(DirectoryLoader):
             self.all_rules.add(rule)
             self.loaded_group.add_rule(rule)
 
-            rule.inherited_platforms.append(self.loaded_group.platform)
+            if self.loaded_group.platforms:
+                rule.inherited_platforms += self.loaded_group.platforms
 
             if self.resolved_rules_dir:
                 output_for_rule = os.path.join(
