@@ -3,9 +3,12 @@ import logging
 import os
 from glob import glob
 
+import ssg.build_yaml
 import ssg.yaml
 import ssg.utils
 
+from ssg.rules import get_rule_path_by_id
+
 
 class Control():
     def __init__(self):
@@ -19,7 +22,7 @@ class Control():
         self.automated = ""
 
     @classmethod
-    def from_control_dict(cls, control_dict, default_level=""):
+    def from_control_dict(cls, control_dict, env_yaml=None, default_level=""):
         control = cls()
         control.id = ssg.utils.required_key(control_dict, "id")
         control.title = control_dict.get("title")
@@ -34,12 +37,38 @@ class Control():
         control.level = control_dict.get("level", default_level)
         control.notes = control_dict.get("notes", "")
         selections = control_dict.get("rules", [])
+
+        product = None
+        product_dir = None
+        benchmark_root = None
+        if env_yaml:
+            product = env_yaml.get('product', None)
+            product_dir = env_yaml.get('product_dir', None)
+            benchmark_root = env_yaml.get('benchmark_root', None)
+            content_dir = os.path.join(product_dir, benchmark_root)
+
         for item in selections:
             if "=" in item:
                 varname, value = item.split("=", 1)
                 control.variables[varname] = value
             else:
-                control.rules.append(item)
+                # Check if rule is applicable to product, i.e.: prodtype has product id
+                if product is None:
+                    # The product was not specified, simply add the rule
+                    control.rules.append(item)
+                else:
+                    rule_yaml = get_rule_path_by_id(content_dir, item)
+                    if rule_yaml is None:
+                        # item not found in benchmark_root
+                        continue
+                    rule = ssg.build_yaml.Rule.from_yaml(rule_yaml)
+                    if rule.prodtype == "all" or product in rule.prodtype:
+                        control.rules.append(item)
+                    else:
+                        logging.info("Rule {item} doesn't apply to {product}".format(
+                            item=item,
+                            product=product))
+
         control.related_rules = control_dict.get("related_rules", [])
         control.note = control_dict.get("note")
         return control
@@ -64,7 +93,7 @@ class Policy():
 
         for node in tree:
             control = Control.from_control_dict(
-                node, default_level=default_level)
+                node, self.env_yaml, default_level=default_level)
             if "controls" in node:
                 for sc in self._parse_controls_tree(node["controls"]):
                     yield sc
