@@ -3,6 +3,7 @@ from __future__ import print_function
 
 import codecs
 import yaml
+import os
 import sys
 import re
 
@@ -153,6 +154,7 @@ def _validate_product_oval_feed_url(contents):
 def open_environment(build_config_yaml, product_yaml):
     contents = open_raw(build_config_yaml)
     contents.update(open_raw(product_yaml))
+    contents["product_dir"] = os.path.dirname(product_yaml)
     _validate_product_oval_feed_url(contents)
     platform_package_overrides = contents.get("platform_package_overrides", {})
     # Merge common platform package mappings, while keeping product specific mappings
@@ -223,12 +225,15 @@ def _strings_to_list(one_or_more_strings):
         return list(one_or_more_strings)
 
 
-def update_yaml_list_or_string(current_contents, new_contents):
+def update_yaml_list_or_string(current_contents, new_contents, prepend=False):
     result = []
     if current_contents:
         result += _strings_to_list(current_contents)
     if new_contents:
-        result += _strings_to_list(new_contents)
+        if prepend:
+            result = _strings_to_list(new_contents) + result
+        else:
+            result += _strings_to_list(new_contents)
     if not result:
         result = ""
     if len(result) == 1:
