@@ -4,8 +4,8 @@ import os
 
 import ssg.controls
 import ssg.build_yaml
-from ssg.environment import open_environment
-from ssg.products import load_product_yaml
+from ssg.yaml import open_environment
+from ssg.products import get_product_yaml
 
 data_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
 controls_dir = os.path.join(data_dir, "controls_dir")
